<?php
namespace Teknisa\CTB\Controllers;

use Teknisa\CTB\Services\Ctb14000EncerramentoExercicioService;
use Zeedhi\Framework\DataSource\ParameterBag;
use Teknisa\Libs\Service\CRUD as CrudService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;

class Ctb14000EncerramentoExercicioController extends \Teknisa\CTB\Controllers\TeknisaCrud {

    protected $dataSourceName = "";
    protected $parameterBagColumns = array();

    public function __construct(CrudService $crudService, ParameterBag $parameterBag, Ctb14000EncerramentoExercicioService $serviceEncerramento){
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
        $this->serviceEncerramento = $serviceEncerramento;
        parent::__construct($crudService, $parameterBag);
    }

    public function findSaldoFiscal(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = "saldempr_contctbl";
        $this->parameterBagColumns = array("P_CDCONTDESTINO", "P_CDEMPRESA", "P_DTSALDEMPR", "NRORG");
        parent::find($request, $response);
    }

    public function findSaldoSocietario(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = "saldosociet_contctbl";
        $this->parameterBagColumns = array("P_CDCONTDESTINO", "P_CDEMPRESA", "P_DTSALDOSOCIET", "NRORG");
        parent::find($request, $response);
    }

    public function verificaCdLote(DTO\Request\Row $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            // Valida se já existe um lote com o mesmo código informado na tela.
            $this->serviceEncerramento->verificaCdLote($row);
        }catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function geraLote(DTO\Request\Row $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            $this->serviceEncerramento->geraLote($row);
        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getMask(DTO\Request\Row $request, DTO\Response $response){
        try{
            $result = $this->serviceEncerramento->getMask();
            $dataSet = new DataSet('dataSetMask', array("CDPICTCCTB" => $result));
            $response->addDataSet($dataSet);
        }catch(\Exception $e){
            $response->setError(new Error($e>getMessage(), $e->getCode()));
        }
    }
    
    public function saveContResul(DTO\Request\DataSet $dataset, DTO\Response $response){
        try {
            $dataset = $dataset->getDataSet()->getRows();
            
            $contas   = $dataset[0];
            $encerramento = $dataset[1];
            
            $result = $this->serviceEncerramento->saveContResul($contas, $encerramento);
    
            $response->addDataSet(new DataSet('saveContResul', $result));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteContResul(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();

            $this->serviceEncerramento->deleteContResul($row);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function finalizaContResul(DTO\Request\DataSet $dataset, DTO\Response $response){
        try {
         
            $dataset = $dataset->getDataSet()->getRows();
            
            $contas   = $dataset[0];
            $encerramento = $dataset[1];
            
   
            $result = $this->serviceEncerramento->finalizaContResul($contas, $encerramento);
    
            $response->addDataSet(new DataSet('finalizaContResul', []));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}
