<?php

namespace Teknisa\CTB\Controllers;

use Teknisa\CTB\Services\Ctb33700ImportacaoLoteContabilService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error; 
use Zeedhi\Framework\DataSource\ParameterBag;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\XLS\Exception;
// use Teknisa\CTB\Modules\ContaPagar\Controllers\TitulosAPagarController;
// use Teknisa\CTB\Modules\ContaReceber\Controllers\TitulosAReceberController;

use Teknisa\CTB\Controllers\TeknisaCrud;

class Ctb33700ImportacaoLoteContabilController extends \Teknisa\CTB\Controllers\TeknisaCrud {
    
    const PADRAO = 'P';
    const METADADOS = 'MD';
    
    protected $service;
    protected $crudService;
    protected $parameterBag;
    protected $nrorg;
    protected $environment;
    
    public function __construct($crudService, ParameterBag $parameterBag, Ctb33700ImportacaoLoteContabilService $service) {
        $this->Ctb33700ImportacaoLoteContabilService = $service;
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
        parent::__construct($crudService, $parameterBag);
    }
    
    public function importPadraoLoteContabil(DTO\Request $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            
            //Lê a tabela de títulos da planilha.
            $table = $this->Ctb33700ImportacaoLoteContabilService->readTable($row, self::PADRAO);
            //Formata dados das linhas
            $table->setRows($this->Ctb33700ImportacaoLoteContabilService->formatTitulos($table, self::PADRAO));
            
            //Valida os dados da tabela.
            $dbColumnValues = array('CDEMPRESA' => $row['CDEMPRESA']);
            $this->Ctb33700ImportacaoLoteContabilService->validateTitulosTable($table, self::PADRAO, $dbColumnValues);
            /*
            
            $rows = $table->getRows();
            $dataSet = $this->service->buildTitulosAPagarDataSet($row['CDEMPRESA'], $rows);
            
           // $this->titulosAPagarController->geraTitulosAPagar($dataSet, $response);
            */
            $response->addDataSet(new DataSet('importPlanilha', array('success' => true)));
        }
        catch(Exception $e){
            $errors = $e->getErrors();
            $response->addDataSet(new DataSet('errors', $errors));
            $response->setError(new Error($e->getMessage(), 500));
        }
        catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function importMetadadosLoteContabil(DTO\Request $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            
            //Lê a tabela de títulos da planilha.
            $table = $this->Ctb33700ImportacaoLoteContabilService->readTable($row, self::METADADOS);
            //Formata dados das linhas
            $table->setRows($this->Ctb33700ImportacaoLoteContabilService->formatTitulos($table, self::METADADOS));
            
            //Valida os dados da tabela.
            $dbColumnValues = array('CDEMPRESA' => $row['CDEMPRESA']);
            $this->Ctb33700ImportacaoLoteContabilService->validateTitulosTable($table, self::METADADOS, $dbColumnValues);
            /*
            
            $rows = $table->getRows();
            $dataSet = $this->service->buildTitulosAPagarDataSet($row['CDEMPRESA'], $rows);
            
           // $this->titulosAPagarController->geraTitulosAPagar($dataSet, $response);
            */
            $response->addDataSet(new DataSet('importPlanilha', array('success' => true)));
        }
        catch(Exception $e){
            $errors = $e->getErrors();
            $response->addDataSet(new DataSet('errors', $errors));
            $response->setError(new Error($e->getMessage(), 500));
        }
        catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
}