<?php
namespace Teknisa\CTB\Factories;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Ctb00001RelatorioFaturamentoCustomizavelFactory extends \Teknisa\CTB\Controllers\TeknisaCrud {

    protected $connection;
    
    const PROCEDURE_GERA_BALANCETE = "GERA_BALANCETE_GERENCIAL";

    public function __construct(EntityManager $entityManager, $environment) {
        $this->connection  = $entityManager->getConnection();
        $this->environment = $environment;
        
    }
    
    public function buscaDadosFaturamento($cdcontctbl, $cdempresa, $cdtipooper, $datafin, $dataini){
        $sql = "SELECT
                    NF.NRORG,
                    EM.CDEMPRESA,
                    EM.NMEMPRESA,
                    NF.CDTIPOOPER,
                    FI.CDAREAGERE AS REGIONAL,
                    NF.CDFILIEMIT AS FILIAL,
                    FI.NRINSJURFILI AS CNPJ_FILIAL,
                    FI.CDINSCESTA AS INSCRICAO_CR,
                    EF.SGESTADO AS ESTADO_UNIDADE,
                    FI.CDFILIFISC AS FILIAL_FISCAL,
                    MAX(CL.NMRAZSOCCLIE) AS RAZAO_SOCIAL_CLIENTE,
                    MAX(CL.CDCLIENTE) AS CNPJ_CLIENTE,
                    NF.NRNOTAFISC AS NRNOTAFSICAL,
                    NF.DTEMISSAO AS DATA_EMISSAO,
                    NF.DTENTRSAID AS DATA_SAIDA,
                    NF.IDENTRSAIDOP AS IDENTIFICACAO_ENTRADA_SAIDA,
                    NF.CDTIPOOPER AS TIPO_OPERACAO,
                    LL.VRLANCAMENT,
                    LL.NRLANCCONT,
                    N.VRNOTAFISC AS VALOR_NF,
                    LL.CDCONTCTBL 
                FROM 
                    NOTAFISCAL NF,
                    FILIAL FI,
                    CLIENTE CL,
                    TIPOOPERAC TP,
                    EMPRESA EM,
                    ENDEFILI EF,
                    (
                        SELECT 
                            SUM(VRLANCAMENT) AS VRLANCAMENT, 
                            CDCONTCTBL,
                            CDEMPRESA,
                            NRLANCCONT
                        FROM LANCCONT LL
                        WHERE 
                            LL.CDEMPRESA = :P_CDEMPRESA
                            AND LL.CDCONTCTBL IN $cdcontctbl
                            AND LL.DTLANCCONT BETWEEN :P_DATAINI AND :P_DATAFIN
                        GROUP BY 
                            CDCONTCTBL,
                            CDEMPRESA,
                            NRLANCCONT
                    ) LL,
                    (
                        SELECT
                            SUM(DECODE(N.IDSISTEMA,'TMP',N.VRPRODUTOS,N.VRNOTAFISC)) AS VRNOTAFISC,
                            NRNOTAFISC,
                            CDFILIAL,
                            CDEMPRESA,
                            NRLANCTBDOC
                        FROM NOTAFISCAL N
                        WHERE 
                            N.CDTIPOOPER IN $cdtipooper AND 
                            N.IDSTATUS IN ('A','E') AND 
                            N.IDENTRSAIDOP IN ('S') AND
                            N.IDSISTEMA IN ('DFE','DFS','TMP','FAI','OPL','VND') AND
                            N.CDEMPRESA = :P_CDEMPRESA AND
                            N.DTEMISSAO BETWEEN :P_DATAINI AND :P_DATAFIN
                        GROUP BY
                            NRNOTAFISC,
                            CDFILIAL,
                            CDEMPRESA,
                            NRLANCTBDOC
                    ) N
                    WHERE 
                        NF.CDEMPRESA = EM.CDEMPRESA AND
                        FI.CDFILIAL = EF.CDFILIAL AND
                        EM.CDEMPRESA = FI.CDEMPRESA AND
                        CL.CDCLIENTE (+) = NF.CDCLIENTE AND
                        TP.CDTIPOOPER = NF.CDTIPOOPER AND
                        TP.IDENTRSAIDOP = NF.IDENTRSAIDOP AND
                        NF.CDFILIEMIT = FI.CDFILIAL AND
                        NF.CDTIPOOPER IN $cdtipooper AND
                        NF.IDSTATUS IN ('A','E') AND
                        NF.IDENTRSAIDOP IN ('S') AND
                        NF.IDSISTEMA IN ('DFE','DFS','TMP','FAI','OPL','VND') AND
                        NF.CDEMPRESA = :P_CDEMPRESA AND
                        LL.NRLANCCONT = NF.NRLANCTBDOC AND
                        LL.CDEMPRESA = NF.CDEMPRESA AND
                        LL.NRLANCCONT = N.NRLANCTBDOC AND
                        LL.CDEMPRESA = N.CDEMPRESA AND
                        NF.CDFILIAL = N.CDFILIAL AND
                        NF.NRNOTAFISC = N.NRNOTAFISC AND
                        NF.DTEMISSAO BETWEEN :P_DATAINI AND :P_DATAFIN
                        GROUP BY LL.CDCONTCTBL,
                        EM.CDEMPRESA,
                        EM.NMEMPRESA,
                        NF.NRORG, 
                        FI.CDAREAGERE ,
                        NF.CDFILIEMIT ,
                        FI.NRINSJURFILI ,
                        FI.CDINSCESTA,
                        EF.SGESTADO ,
                        FI.CDFILIFISC ,
                        CL.NMRAZSOCCLIE,
                        CL.CDCLIENTE ,
                        NF.NRNOTAFISC ,
                        NF.DTEMISSAO ,
                        NF.DTENTRSAID ,
                        DECODE(NF.IDSISTEMA,'TMP',NF.VRPRODUTOS,NF.VRNOTAFISC) ,
                        NF.IDENTRSAIDOP,
                        NF.CDTIPOOPER,
                        LL.VRLANCAMENT,
                        LL.NRLANCCONT,
                        N.VRNOTAFISC
                    HAVING 
                        LL.VRLANCAMENT <> N.VRNOTAFISC
                    ORDER BY 1,2,3";

        $params = array(
            "P_DATAFIN" => $datafin,
            "P_DATAINI" => $dataini,
            "P_CDEMPRESA" => $cdempresa
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
}