<?php
namespace Teknisa\CTB\Factories;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Teknisa\Model\Geral\Entities\Lista;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Ctb14000EncerramentoExercicioFactory {

    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }

    public function verificaCdLote($params) {
        $sql = "SELECT 1 FROM CAPALOTE WHERE CDLOTE = :CDLOTE AND CDEMPRESA = :CDEMPRESA AND NRORG = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetch();
    }
    
    public function geraLote($row){
        /* Variáveis que definem os parâmtros da procedure para gerar o lote de encerramento */
        $cdempresa = $row['CDEMPRESA']; // Empresa do encerramento
        $cdcontctbl = $row['CDCONTCTBL']; // Conta de destino do resultado, normalmente é uma conta do PL
        $cdhistpadr = $row['CDHISTPADR']; // Código do histórico que será buscado a descrição dos lançamentos contábeis
        $dtencerramento = $row['DTENCERRAMENTO']; // Data do encerramento, último dia do mês.
        $cdlote = $row['CDLOTE']; // Código do lote de encerramento
        $nrlista = null; // Este campo deve ser nulo, pois na procedure será usado para fazer a distinção do Delphi e Zeedhi
        $idtiposaldo = $row['IDTIPOSALDO']; // Saldo Fiscal ou Societário, F ou S
        
        $procGeraLote = new StoredProcedure($this->connection, "TRANSFERE_SALDO_CONTCTBL");
        $procGeraLote->addParam(new Param('P_CDEMPRESA', Param::PARAM_INPUT, $cdempresa));
        $procGeraLote->addParam(new Param('P_CDCONTADEST', Param::PARAM_INPUT, $cdcontctbl));
        $procGeraLote->addParam(new Param('P_CDHISTPADR', Param::PARAM_INPUT, $cdhistpadr));
        $procGeraLote->addParam(new Param('P_DTSALDO', Param::PARAM_INPUT, $dtencerramento));
        $procGeraLote->addParam(new Param('P_CDLOTE', Param::PARAM_INPUT, $cdlote));
        $procGeraLote->addParam(new Param('P_NRLISTA', Param::PARAM_INPUT, $nrlista));
        $procGeraLote->addParam(new Param('P_IDENCERRAMEN', Param::PARAM_INPUT, $idtiposaldo));
        $procGeraLote->execute();
    }
    
    public function getMask($arrayNrOrg){
        $sql = "SELECT CDPICTCCTB FROM PARAMGERAL WHERE NRORG = :NRORG";
        $statement = $this->connection->prepare($sql);
        $statement->execute($arrayNrOrg);
        return $statement->fetch();
    }
    
    public function buscaDatasContResul($params) {

        // Query a ser executada.
        $sql = "SELECT DTINICCONT, DTFINACONT, DTINICLANC, DTFINALANC
                FROM CONTOPECTBL
                WHERE CDEMPRESA = :P_CDEMPRESA
                AND NRORG = :NRORG
                "
                ;

        // Executa o statement com os paràmetros passados.
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function buscaContResul($params) {

        // Query a ser executada.
        $sql = "SELECT S.CDCONTCTBL, C.NMCONTCTBL, ROUND(S.VRSALDEMPR, 2) VRSALDEMPR1, TO_CHAR(ROUND(S.VRSALDEMPR, 2)) VRSALDEMPR
                FROM SALDEMPR S, CONTCTBL C
                WHERE S.CDCONTCTBL = C.CDCONTCTBL
                AND ROUND(S.VRSALDEMPR, 2) <> 0
                AND S.CDEMPRESA = :P_CDEMPRESA
                AND S.DTSALDEMPR = :P_DTDATA
                AND C.IDTIPOCONT = 'A'
                AND C.CDCONTCTBL LIKE :P_CDCONTCTBL || '%'
                AND C.CDCONTCTBL <> :P_CDCONTDEST
                AND C.IDCONTCCUS <> 'R'
                AND (C.IDNATUCONT = '04' OR C.IDNATUCONT IS NULL)
                AND 'F' = :P_IDSALDO
                AND C.IDTIPOCONT = 'A'
                AND C.NRORG = :NRORG
                UNION
                
                SELECT S.CDCONTCTBL, C.NMCONTCTBL, ROUND(S.VRSALDOSOCIET, 2) VRSALDEMPR1, TO_CHAR(ROUND(S.VRSALDOSOCIET, 2)) VRSALDEMPR
                FROM SALDOSOCIET S, CONTCTBL C
                WHERE S.CDCONTCTBL = C.CDCONTCTBL
                AND ROUND(S.VRSALDOSOCIET, 2) <> 0
                AND S.CDEMPRESA = :P_CDEMPRESA
                AND S.DTSALDOSOCIET = :P_DTDATA
                AND C.IDTIPOCONT = 'A'
                AND C.CDCONTCTBL LIKE :P_CDCONTCTBL || '%'
                AND C.CDCONTCTBL <> :P_CDCONTDEST
                AND C.IDCONTCCUS <> 'R'
                AND (C.IDNATUCONT = '04' OR C.IDNATUCONT IS NULL)
                AND 'S' = :P_IDSALDO
                AND C.NRORG = :NRORG
                ORDER BY 1"
                ;

        // Executa o statement com os paràmetros passados.
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public static function insereLista($nrorg, $nrlista, $cdlista)
        {
        $lista = new Lista();
        $lista->setNrorg($nrorg);
        $lista->setNrlista($nrlista);
        $lista->setCdlista($cdlista);

        return $lista;
    }
    
      public function insereLista2($nrorg, $nrlista, $cdlista){
        /** @var $params */
        $params = array();
        $params['P_NRORG'] = $nrorg;
        $params['P_NRLISTA'] = $nrlista;
        $params['P_CDLISTA'] = $cdlista;

        $sql = "INSERT INTO LISTA
                    (NRORG, NRLISTA, CDLISTA)
                VALUES
                    (:P_NRORG, :P_NRLISTA, :P_CDLISTA)";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function execProcTRANSFERE_SALDO_CONTCTBL(
        $cdempresa, $cdcontadest, $cdhistpadr, $dtsaldo, $cdlote, $nrlista, $idencerramen
    ){
        $procedure = new StoredProcedure($this->connection, 'TRANSFERE_SALDO_CONTCTBL');
        $procedure->addParam(new Param('P_CDEMPRESA'    , Param::PARAM_INPUT , $cdempresa    , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCONTADEST'  , Param::PARAM_INPUT , $cdcontadest  , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDHISTPADR', Param::PARAM_INPUT , $cdhistpadr, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTSALDO' , Param::PARAM_INPUT , $dtsaldo , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDLOTE' , Param::PARAM_INPUT , $cdlote , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLISTA'  , Param::PARAM_INPUT , $nrlista  , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDENCERRAMEN'  , Param::PARAM_INPUT , $idencerramen  , Param::PARAM_TYPE_STR));
        $procedure->execute();

    }
    
    public function deleteLista($params) {
        // Executa a query para remover rateios associados.
        //$this->deleteRateiosLancamento($params);

        // Query a ser executada.
        $sql = "DELETE FROM LISTA WHERE NRLISTA = :NRLISTA AND NRORG = :NRORG";

        // Prepara o statement da query para execução.
        $statement = $this->connection->prepare($sql);

        // Executa o statement com os paràmetros passados.
        $statement->execute($params);
    }
}