<?php

    namespace Teknisa\CTB\Factories;

use Doctrine\ORM\EntityManager;

class Ctb33700ImportacaoLoteContabilFactory {
    
    protected $entityManager;
    protected $connection;  

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }
    
    public function buscaLote($params) {
        // Query a ser executada.
        $sql = "SELECT CDEMPRESA,  CDLOTE,
                       DTINICLOTE, DTFINALOTE,
                       DSOBSELOTE, IDLOTE
                FROM CAPALOTE
                WHERE (CDEMPRESA = :CDEMPRESA) AND
                      (CDLOTE    = :CDLOTE)";

        // Prepara o statement da query para execução.
        $statement = $this->connection->prepare($sql);

        // Executa o statement com os paràmetros passados.
        $statement->execute($params);
        return $statement->fetchAll();

    }
    
    public function getCentCustDeb($cdfilialdeb, $cdcentcustdeb){
        $params = array();
        $params['CDFILIALDEB'] = $cdfilialdeb;
        $params['CDCENTCUSTDEB'] = $cdcentcustdeb;
        $sql = "SELECT * FROM CENTCUSTFIL WHERE CDFILIAL = :CDFILIALDEB AND CDCENTCUST = :CDCENTCUSTDEB";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getFilialDeb($cdempresa, $cdfilialdeb){
        $params = array();
        $params['CDFILIALDEB'] = $cdfilialdeb;
        $params['CDEMPRESA'] = $cdempresa;
        $sql = "SELECT * FROM FILIAL WHERE CDFILIAL = :CDFILIALDEB AND CDEMPRESA = :CDEMPRESA";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getCentCustCred($cdcentcust, $cdfilial){
        $params = array();
        $params['CDFILIAL'] = $cdfilial;
        $params['CDCENTCUST'] = $cdcentcust;
        $sql = "SELECT * FROM CENTCUSTFIL WHERE CDFILIAL = :CDFILIAL AND CDCENTCUST = :CDCENTCUST";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getFilialCred($cdempresa, $cdfilial){
        $params = array();
        $params['CDFILIAL'] = $cdfilial;
        $params['CDEMPRESA'] = $cdempresa;
        $sql = "SELECT * FROM FILIAL WHERE CDFILIAL = :CDFILIAL AND CDEMPRESA = :CDEMPRESA";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getContContabilDeb($cdcontctbl){
        $params = array();
        $params['CDCONTCTBLDEB'] = $cdcontctbl;
        $sql = "SELECT * FROM CONTCTBL WHERE CDCONTCTBL = :CDCONTCTBLDEB";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getContContabilCred($cdcontctbl){
        $params = array();
        $params['CDCONTCTBL'] = $cdcontctbl;
        $sql = "SELECT * FROM CONTCTBL WHERE CDCONTCTBL = :CDCONTCTBL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getCodHistPadrao($cdhistpadr){
        $params = array();
        $params['CDCONTHISTPADR'] = $cdhistpadr;
        $sql = "SELECT * FROM HISTPADR WHERE CDHISTPADR = :CDCONTHISTPADR";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function insertLancloteNegativo($params) {
        $sql = "INSERT INTO LANCLOTE
                            (
                            CDEMPRESA,  CDLOTE, 
                            NRLANCLOTE, NRSEQLANLOTE, 
                            CDCONTCTBL, CDHISTPADR,
                            DTLANCLOTE, NRDOCULOTE,
                            DSHISTLOTE, VRLANCLOTE,
                            IDLLOTCOME, CDLLOTCOME
                            )
                        values
                            (
                            :CDEMPRESA,     :CDLOTE, 
                            :NRLANCLOTE,    :NRSEQLANLOTE, 
                            :CDCONTCTBLDEB, :CDCONTHISTPADR,
                            :DTLANCLOTE,    :NRDOCU,
                            :DSHISTLOTE,    :VRLANINC,
                            :IDCOMERCIAL,   :CDCOMERCIAL
                             )";

        $std = $this->connection->prepare($sql);
        return $std->execute($params);
    }
    
    public function insertLanclotePositivo($params) {
        $sql = "INSERT INTO LANCLOTE
                            (
                            CDEMPRESA,  CDLOTE, 
                            NRLANCLOTE, NRSEQLANLOTE, 
                            CDCONTCTBL, CDHISTPADR,
                            DTLANCLOTE, NRDOCULOTE,
                            DSHISTLOTE, VRLANCLOTE,
                            IDLLOTCOME, CDLLOTCOME
                            )
                        values
                            (
                            :CDEMPRESA,   :CDLOTE, 
                            :NRLANCLOTE,  :NRSEQLANLOTE, 
                            :CDCONTCTBL,  :CDCONTHISTPADR,
                            :DTLANCLOTE,  :NRDOCU,
                            :DSHISTLOTE,  :VRLANINC,
                            :IDCOMERCIAL, :CDCOMERCIAL
                             )";

        $std = $this->connection->prepare($sql);
        return $std->execute($params);
    }
    
    public function insertRatccusNegativo($params) {
        $sql = "INSERT INTO RATECCUS
                            (
                            CDEMPRESA,  CDLOTE, 
                            NRLANCLOTE, NRSEQLANLOTE, 
                            CDFILIAL,   CDCENTCUST,
                            VRRATECCUS, DTCOMPRATECCUS
                            )
                        values
                            (
                            :CDEMPRESA,   :CDLOTE, 
                            :NRLANCLOTE,  :NRSEQLANLOTE, 
                            :CDFILIALDEB, :CDCENTCUSTDEB,
                            :VRLANINC,    :DTLANCLOTE
                             )";

        $std = $this->connection->prepare($sql);
        return $std->execute($params);
    }
    
    public function insertRatccusPositivo($params) {
        $sql = "INSERT INTO RATECCUS
                            (
                            CDEMPRESA,  CDLOTE, 
                            NRLANCLOTE, NRSEQLANLOTE, 
                            CDFILIAL,   CDCENTCUST,
                            VRRATECCUS, DTCOMPRATECCUS
                            )
                        values
                            (
                            :CDEMPRESA,  :CDLOTE, 
                            :NRLANCLOTE, :NRSEQLANLOTE, 
                            :CDFILIAL,   :CDCENTCUST,
                            :VRLANINC,   :DTLANCLOTE
                             )";

        $std = $this->connection->prepare($sql);
        return $std->execute($params);
    }
   /* 
    public function insertLanclote($nrorg, $nrseqlanlote, $nrlanclote, $cdempresa, $cdlote, $nrdoculote, $dtlanclote, $cdcontctbl,
         $cdhistpadr, $valorabs, $cdcontcome, $idllotcome, $dshistlanc,$dataatual)
        {
            $params = array();
            $params['NRORG']   = $nrorg;
            $params['NRSEQLANLOTE']   = $nrseqlanlote;
            $params['NRLANCLOTE']   = $nrlanclote;
            $params['CDEMPRESA']   = $cdempresa;
            $params['CDLOTE']   = $cdlote;
            $params['NRDOCULOTE']   = $nrdoculote;
            $params['DTLANCLOTE']   = $dtlanclote;
            $params['CDCONTCTBL']   = $cdcontctbl;
            $params['CDHISTPADR']   = $cdhistpadr;
            $params['VRLANCLOTE']   = $valorabs;
            $params['IDLLOTCOME']   = $idllotcome;
            $params['CDLLOTCOME']   = $cdcontcome;
            $params['DSHISTLOTE']   = $dshistlanc;

            $sql = "INSERT INTO LANCLOTE (
                                    NRORG,
                                    NRSEQLANLOTE,
                                    NRLANCLOTE,
                                    CDEMPRESA,
                                    CDLOTE,
                                    NRDOCULOTE,
                                    DTLANCLOTE,
                                    CDCONTCTBL,
                                    CDHISTPADR,
                                    VRLANCLOTE,
                                    IDLLOTCOME,
                                    CDLLOTCOME,
                                    DSHISTLOTE
                                ) VALUES (
                                    :NRORG,
                                    :NRSEQLANLOTE,
                                    :NRLANCLOTE,
                                    :CDEMPRESA,
                                    :CDLOTE,
                                    :NRDOCULOTE,
                                    :DTLANCLOTE,
                                    :CDCONTCTBL,
                                    :CDHISTPADR,
                                    :VRLANCLOTE,
                                    :IDLLOTCOME,
                                    :CDLLOTCOME,
                                    :DSHISTLOTE
                                )";
                       
        // Prepara o statement da query para execução.
        $statement = $this->connection->prepare($sql);

        // Executa o statement com os paràmetros passados.
        $statement->execute($params);
    }
    */
}