<?php
namespace Teknisa\CTB\Modules\Saldos\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Ctb44200RazaoGerencialService {

    protected $entityManager;
    protected $environment;
    protected $newCode;
    protected $connection;

    public function __construct(EntityManager $entityManager, Environment $environment,NewCode $newCode) {
        $this->entityManager = $entityManager;
        $this->environment   = $environment;
        $this->newCode = $newCode;
        $this->operador = $this->environment->getCdOperador();
        $this->connection = $entityManager->getConnection();
    }

    public function geraRazaoGerencialNovo($row) {
        try{
            
            foreach($row as $params){
                switch ($params["name"]) {
                    case "DTINICIAL":
                        $datainic = $params["value"];
                    break;
                    case "DTFINAL":
                        $datafin =  $params["value"];
                    break;
                    case "CDCONTINI":
                        $cdcontini = $params["value"];
                    break;
                    case "CDCONTFIN":
                        $cdcontfin = $params["value"];
                    break;
                    case "CDEMPRESA":
                        $cdempresa = $params["value"];
                    break;
                    case "CDFILIAL":
                        $cdfilial = $params["value"];
                    break;
                    case "CDAREAGERE":
                        $cdregional = $params["value"];
                    break;
                    case "CDCENTCUST":
                        $cdcentcust = $params["value"];
                    break;
                }
            }
            
            if(empty($cdempresa)){
                $cdempresa = 'T';
            } 
            if(empty($cdregional)){
                $cdregional = 'T';
            } 
            if(empty($cdfilial)){
                $cdfilial = 'T';
            } 
            if(empty($cdcentcust)) {
                $cdcentcust = 'T';
            }

            $nrorg = $this->environment->getNrOrgTrab();
            $nrctbtemp = $this->newCode->getCodeByTableName('CTBTEMP', $nrorg, 1, 6);
            
            //validacao para acessar apenas a filial que o operador tem permissao
            $operador = $this->operador = $this->environment->getCdOperador();
            $validacaoOperador = $this->validaOperadorLogado($cdfilial,$operador);
            if(empty($validacaoOperador) || $cdfilial == ''){
                $cdfilial = 'T';
            }elseif(empty($validacaoOperador)){
                throw new \Exception("Operação bloqueada. Operador não relacionado a filial informada! Informe a filial correta.");
            }

            $this->execProcGeraRazaoGerTemp(
               $cdempresa, $datainic, $datafin, $nrctbtemp, $cdregional, $cdfilial, $cdcentcust, $cdcontini, $cdcontfin
            );
        } catch (\Exception $e) {
            throw $e;
        }

        return $nrctbtemp;
    }

    public function execProcGeraRazaoGerTemp(
        $cdempresa, $datainic, $datafin, $nrctbtemp, $cdregional, $cdfilial, $cdcentcust, $cdcontini, $cdcontfin
        
    ){
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'GERA_RAZAOGER_SAP');
        $procedure->addParam(new Param('P_CDEMPRESA'  , Param::PARAM_INPUT , $cdempresa  , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTINICIAL'   , Param::PARAM_INPUT , $datainic   , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTFINAL'    , Param::PARAM_INPUT , $datafin    , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRCTBTEMP'  , Param::PARAM_INPUT , $nrctbtemp  , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDAREAGERE' , Param::PARAM_INPUT , $cdregional , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIAL'   , Param::PARAM_INPUT , $cdfilial   , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCENTCUST' , Param::PARAM_INPUT , $cdcentcust , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCONTINI'  , Param::PARAM_INPUT , $cdcontini  , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCONTFIM'  , Param::PARAM_INPUT , $cdcontfin  , Param::PARAM_TYPE_STR));
        $procedure->execute();
    }
    
    public function validaOperadorLogado($cdfilial,$operador) {
		$sql = "SELECT CDFILIAL
		        FROM FILIOPER FO
		        WHERE FO.CDFILIAL = :CDFILIAL
		        AND FO.CDOPERADOR = :CDOPERADOR";

		$params = array("CDFILIAL"=>$cdfilial,
		                "CDOPERADOR"=>$operador);
						
	    $std = $this->connection->prepare($sql);
		$std->execute($params);
		return $std->fetchAll();
	}

}