<?php

namespace Teknisa\CTB\Services;

use Zeedhi\Framework\DTO\Request\DataSet;
use Teknisa\Libs\Util\XLS\FileLoader;
use Teknisa\Libs\Util\XLS\FileReader;
use Teknisa\Libs\Util\Environment;
use Teknisa\CTB\Services\ImportacaoLoteError;
use Teknisa\CTB\Factories\Ctb33700ImportacaoLoteContabilFactory;
use Teknisa\Libs\Util\XLS\Table;
use Teknisa\Libs\Util\XLS\Exception;
use Zeedhi\Framework\DataSource;
use PHPExcel_Cell;
use PHPExcel_Shared_Date;
use PhpOffice\PhpSpreadsheet;

class Ctb33700ImportacaoLoteContabilService{ 
    
    const PADRAO = 'P';
    // const METADADOS = 'MD';
    
    const CAMPOS_LENGTH = 4;
    const SEQUENCIAL_LENGTH = 6;
    const CONTCTBL_LENGTH = 20;
    
    const IMPORT_EMPTY_CELL_VALUE_ERROR = 'IMPORT_EMPTY_CELL_VALUE_ERROR';
    const IMPORT_NOT_FOUND_CELL_ERROR = 'IMPORT_NOT_FOUND_CELL_ERROR';
    const IMPORT_NOT_FOUND_TYPE_CELL_ERROR = 'IMPORT_NOT_FOUND_TYPE_CELL_ERROR';
    const IMPORT_NOT_RELATED_CELL_ERROR = 'IMPORT_NOT_RELATED_CELL_ERROR';
    const IMPORT_ROW_EXISTS_ERROR = 'IMPORT_ROW_EXISTS_ERROR';
    const IMPORT_FILE_FORMAT_ERROR = 'IMPORT_FILE_FORMAT_ERROR';
    const IMPORT_ROW_CENTCUST_ERROR = 'IMPORT_ROW_CENTCUST_ERROR';
    const IMPORT_ROW_AJUSTE_ERROR = 'IMPORT_ROW_AJUSTE_ERROR';
    const IMPORT_ROW_TIPOAJUSTECAD_ERROR = 'IMPORT_ROW_TIPOAJUSTECAD_ERROR';
    const IMPORT_ROW_TIPOAJUSTE_ERROR = 'IMPORT_ROW_TIPOAJUSTE_ERROR';
    const IMPORT_ROW_DUPLICATE_ERROR = 'IMPORT_ROW_DUPLICATE_ERROR';
    
    protected $entityManager;
    protected $environment;
    protected $Ctb33700ImportacaoLoteContabilFactory;
    private $titleType;
    protected $columnNames; //nome das colunas da planilha
    protected $nrorg;
    
    public function __construct($entityManager, Environment $environment, Ctb33700ImportacaoLoteContabilFactory $Ctb33700ImportacaoLoteContabilFactory) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->Ctb33700ImportacaoLoteContabilFactory = $Ctb33700ImportacaoLoteContabilFactory;
        $this->nrorg = $this->environment->getNrOrgTrab();
        
    }
     /*** ------------------------------- LEIAUTE PADRÃO -------------------------------
          001 | 06 posições | Sequencial
          007 | 08 posições | Data de Lançamento AAAAMMDD
          015 | 04 posições | Centro de custo a débito
          019 | 04 posições | Filial a débito
          023 | 04 posições | Centro de custo a débito
          027 | 04 posições | Filial a crédito
          031 | 08 posições | Número do documento
          039 | 20 posições | Conta Contábil a Débito
          059 | 20 posições | Conta Contábil a Crédito
          079 | 04 posições | Código do Histórico Padrão
          083 |200 posições | Descrição Histórico Padrão
          283 | 16 posições | Valor do lançamento
          299 | 01 posições | Identificador Comercial
          300 | 21 posições | Código Comercial
         }
         
    ***/
    
    public function verificaLote($row){
        //Verifica se o Lote já existe
            $existeCampoLote = $this->Ctb33700ImportacaoLoteContabilFactory->buscaLote(array(
            "CDLOTE" => $row['CDLOTE'],
            "CDEMPRESA" => $row['CDEMPRESA']
        ));
        if ($existeCampoLote !== false) {
            throw new \Exception("Lote informado já existe.");
        }
    }
    
    private function existsFieldOnDB($value, $cdempresa, $columnName, $titleType, $valores){
        if($titleType === self:: PADRAO || $titleType === self:: METADADOS){
            return self::buscaCampos($value, $cdempresa, $columnName, $valores);
        }
        return true;
    }
    
    private function buscaCampos($value, $cdempresa, $columnName, $valores){
               // var_dump($value, $cdempresa, $columnName, $valores);die;
        switch($columnName){
            case 'CENTRO DE CUSTO A DÉBITO':
                return ($this->Ctb33700ImportacaoLoteContabilFactory->getCentCustDeb($value, $valores["FILIAL A DÉBITO"], $this->nrorg));
                break;
            case 'FILIAL A DÉBITO':
                return ($this->Ctb33700ImportacaoLoteContabilFactory->getFilialDeb($value, $cdempresa));
                break;
            case 'CENTRO DE CUSTO A CRÉDITO':
                return ($this->Ctb33700ImportacaoLoteContabilFactory->getCentCustCred($value, $valores["FILIAL A CRÉDITO"]));
                break;
            case 'FILIAL A CRÉDITO':
                return ($this->Ctb33700ImportacaoLoteContabilFactory->getFilialCred($value, $cdempresa));
                break;
            case 'CONTA CONTÁBIL A DÉBITO':
                return ($this->Ctb33700ImportacaoLoteContabilFactory->getContContabilDeb($value));
                break;
            case 'CONTA CONTÁBIL A CRÉDITO':
                return ($this->Ctb33700ImportacaoLoteContabilFactory->getContContabilCred($value));
                break;
            case 'CÓDIGO DO HISTÓRICO PADRÃO':
                return ($this->Ctb33700ImportacaoLoteContabilFactory->getCodHistPadrao($value));
                break;
            default:
                return false;
        }
    }
    
    private function formatData($data){
        // Alteração de depreciação do PHPExcel - Nas versões acima de 70400
        if ((PHP_VERSION_ID >= 70400) && (gettype($data) == 'integer'))
            $data =  PhpSpreadsheet\Shared\Date::excelToDateTimeObject($data)->format('d/m/Y');
        elseif(gettype($data) == 'double')
            $data =  date('d/m/Y', PHPExcel_Shared_Date::ExcelToPHP($data + 1.0));
        
        return $data;
    }
    
    private function getColumnNames($titleType){
        $columnNames = array();
        if($titleType == self::PADRAO /*|| $titleType == self::METADADOS*/){
            $columnNames = array(
                'SEQUENCIAL', 'DATA DE LANÇAMENTO', 'CENTRO DE CUSTO A DÉBITO', 'FILIAL A DÉBITO',
                'CENTRO DE CUSTO A CRÉDITO', 'FILIAL A CRÉDITO', 'NÚMERO DO DOCUMENTO', 'CONTA CONTÁBIL A DÉBITO',
                'CONTA CONTÁBIL A CRÉDITO', 'CÓDIGO DO HISTÓRICO PADRÃO', 'DESCRIÇÃO HISTÓRICO PADRÃO',
                'VALOR DO LANÇAMENTO', 'IDENTIFICADOR COMERCIAL', 'CÓDIGO COMERCIAL'
            );
        } /*elseif($titleType == self::METADADOS){
            $columnNames = array(
                
            );
        }*/
        return $columnNames;
    }
       
    public function formatTitulos(Table $table, $titleType){
        $columnsReference = $table->getColumnsReference();
        $tableRows = $table->getRows();
        $columnsNumber = $table->getNumberOfColumns();
        foreach ($tableRows as &$row) {
            for($column = 0; $column < $columnsNumber; $column++){
                // Alteração de depreciação do PHPExcel - Nas versões acima de 70400
                if(PHP_VERSION_ID >= 70400)
                     $columnLetterIndex = PhpSpreadsheet\Cell\Coordinate::stringFromColumnIndex($column + 1);
                else $columnLetterIndex = PHPExcel_Cell::stringFromColumnIndex($column);
                
                // $columnName = $columnsReference[$columnLetterIndex];
                // if(!self::fieldNeedAnotherRowData($columnName)){
                //     $row['content'][$columnName] = self::formatValue($row['content'][$columnName], $columnName, $titleType); 
                // }
                // else{
                //     $row['content'][$columnName] = self::formatValue($row['content'][$columnName], $columnName, $titleType, $row['content']);
                // }
            }
        }
        return $tableRows;
    }
    
    private function formatValue($value, $columnName, $titleType, $currentRow = null){
        if($titleType == self::PADRAO/* || $titleType == self::METADADOS*/){
            switch($columnName){
                case 'SEQUENCIAL':
                    $value = self::leftZeroFill($value, self::SEQUENCIAL_LENGTH);
                    break;
                case 'DATA DE LANÇAMENTO':
                    $value = self::formatData($value);
                    break;
                case 'CENTRO DE CUSTO A DÉBITO':
                case 'CENTRO DE CUSTO A CRÉDITO':
                case 'FILIAL A DÉBITO':
                case 'FILIAL A CRÉDITO':
                    $value = self::leftZeroFill($value, self::CAMPOS_LENGTH);
                    break;
                case 'CONTA CONTÁBIL A DÉBITO':
                case 'CONTA CONTÁBIL A CRÉDITO':
                    $value = self::leftZeroFill($value, self::CONTCTBL_LENGTH);
                    break;
                case 'VALOR DO LANÇAMENTO':
                    $value = str_replace(',', '.', $value);
                    $value = floatval($value);
                    break;
            }
        }
        return $value;
    }
    
    private function leftZeroFill($string, $length){
        $string = str_pad(strval($string), $length, '0', STR_PAD_LEFT); 
        return $string;
    }
    
    public function readTable($row, $titleType){
        $file = $row['FILE'];
        $sheetsToLoad = array('Plan1');
        $xlsFile = FileLoader::loadFromBase64($file['content'], $file['name'], $sheetsToLoad);
        if(!$xlsFile){
            throw new \Exception(self::IMPORT_LOAD_FILE_ERROR);
        }
        $worksheet = $xlsFile->getActiveSheet();
        $columnNames = self::getColumnNames($titleType);
        $table = FileReader::readWorksheetTable($worksheet, $columnNames);
        return $table;
    }
    
    private function getTituloSearchColumnNames($titleType){
        $seachColumnNames = array();
        if($titleType == self::PADRAO /*|| $titleType == self::METADADOS*/){
            $seachColumnNames = array(
                'CENTRO DE CUSTO A DÉBITO', 'FILIAL A DÉBITO',
                'CENTRO DE CUSTO A CRÉDITO', 'FILIAL A CRÉDITO', 'CONTA CONTÁBIL A DÉBITO',
                'CONTA CONTÁBIL A CRÉDITO', 'CÓDIGO DO HISTÓRICO PADRÃO',
                'VALOR DO LANÇAMENTO', 'IDENTIFICADOR COMERCIAL'
            );
        }
        return $seachColumnNames;
    }
    
    private function getTituloOptionalColumnNames($titleType){
        $optionalColumnNames = array();
        if($titleType == self::PADRAO/* || $titleType == self::METADADOS*/){
            $optionalColumnNames = array('SEQUENCIAL', 'NÚMERO DO DOCUMENTO', 'DESCRIÇÃO HISTÓRICO PADRÃO', 'CÓDIGO COMERCIAL'); 
        }
        return $optionalColumnNames;
    }
    
    private function isFieldToSearch($columnName, $titleType){
        return in_array($columnName, self::getTituloSearchColumnNames($titleType));
    }
    
    private function isFieldValueOptional($columnName, $titleType, $valores){
        return in_array($columnName, self::getTituloOptionalColumnNames($titleType));
    }
    
    public function validateTitulosTable(Table $table, $titleType, $dbColumnValues){
        $errors = array();
        $columnsReference = $table->getColumnsReference();
        $tableRows = $table->getRows();
        $columnsNumber = $table->getNumberOfColumns();
        foreach ($tableRows as $row) {
            for($column = 0; $column < $columnsNumber; $column++){
                $columnLetterIndex = PHPExcel_Cell::stringFromColumnIndex($column);
                $columnName = $columnsReference[$columnLetterIndex];
                $value = $row['content'][$columnName];
                $valores = $row['content'];
                $error = self::validateTituloFieldValue($valores, $value, $row['index'], $columnLetterIndex, $columnName, $titleType, $dbColumnValues);
               
                if($error !== NULL){
                    array_push($errors, $error->toArray());
                }
            }
            
            // $error = self::validateNotaFiscal($row['content'], $row['index'], $titleType);
            // if($error !== NULL) array_push($errors, $error->toArray());
            // $error = self::validateCentCust($row['content'], $row['index'], $dbColumnValues, $titleType);
            // if($error !== NULL) array_push($errors, $error->toArray());
        }
        if(count($errors) > 0){
            throw new Exception($errors, self::IMPORT_FILE_FORMAT_ERROR);
        }
    }
    
    private function validateTituloFieldValue($valores, $value, $row, $column, $columnName, $titleType, $dbColumnValues){
        $error = NULL;

        if($value === NULL && !self::isFieldValueOptional($columnName, $titleType, $valores)){
            $error = new ImportacaoLoteError(self::IMPORT_EMPTY_CELL_VALUE_ERROR, $row, $column, $columnName);
        }else{
            if(self::isFieldToSearch($columnName, $titleType)){
                $data = self::existsFieldOnDB($value, $dbColumnValues['CDEMPRESA'], $columnName, $titleType, $valores);
                if(!$data){
                    $error = new ImportacaoLoteError(self::IMPORT_NOT_FOUND_CELL_ERROR, $row, $column, $columnName);
                }
            }
            //ele vai entrar aqui se o value for diferente de null, se o campo for opcional e se não for um campo para pesquisa
            else{
                // Se a coluna é igual ao centro de custo e o Centro de Custo a Debito está preenchido, mas a filial a débito está vazio
                if($columnName == 'CENTRO DE CUSTO A DÉBITO' && (!empty($valores['CENTRO DE CUSTO A DÉBITO']) && empty($valores['FILIAL A DÉBITO']))){
                    // Envia para a função que verifica se os dados do Centro de Custo estão corretos
                    $retorno = $this->Ctb33700ImportacaoLoteContabilFactory->getCentCustDeb($value, $valores['FILIAL A DÉBITO']);
                    // faz a inserçao dos dados de centro de custo a debito
                    $insert = $this->Ctb33700ImportacaoLoteContabilFactory->insertLancloteNegativo($valores['CENTRO DE CUSTO A DÉBITO']);
                     //verifica se retornou false
                    if(!$retorno){
                         //exibe mensagem de erro
                        $error = new ImportacaoLoteError(self::IMPORT_NOT_FOUND_CELL_ERROR, $row, $column, $columnName);
                    }
                    // e a coluna é igual ao centro de custo e o Centro de Custo a Credito está preenchido, mas a filial a credito está vazio
                } elseif ($columnName == 'CENTRO DE CUSTO A CRÉDITO' && (empty($valores['CENTRO DE CUSTO A CRÉDITO']) && !empty($valores['FILIAL A CRÉDITO']))){
                    // Envia para a função que verifica se os dados do Centro de Custo estão corretos
                    $retorno = $this->Ctb33700ImportacaoLoteContabilFactory->getCentCustCred($value, $$valores["FILIAL A CRÉDITO"]);
                    // faz a inserçao dos dados de centro de custo a credito
                    $insert = $this->Ctb33700ImportacaoLoteContabilFactory->insertLanclotePositivo($valores['CENTRO DE CUSTO A CRÉDITO']);
                    //verifica se retornou false
                    if(!$retorno){
                        //exibe mensagem de erro
                        $error = new ImportacaoLoteError(self::IMPORT_NOT_FOUND_CELL_ERROR, $row, $column, $columnName);
                    }
                    // verifica se possui valor na coluna filial a debito para fazer o insert
                } if($columnName == 'FILIAL A DÉBITO' && (!empty($valores['FILIAL A DÉBITO']))){
                    $insert = $this->Ctb33700ImportacaoLoteContabilFactory->insertRatccusNegativo($value, $valores['FILIAL A DÉBITO']);
                }
                // verifica se possui valor na coluna filial a credito para fazer o insert
                  else if($columnName == 'FILIAL A CRÉDITO' && (!empty($valores['FILIAL A CRÉDITO']))){
                    $insert = $this->Ctb33700ImportacaoLoteContabilFactory->insertRatccusPositivo($value, $valores['FILIAL A CRÉDITO']);
                }
            }
        }
        
        return $error;
    }
    
    /**{------------------------------- LEIAUTE METADADOS -------------------------------
          001 a 004 | 04 posições | Código da Empresa (NÃO UTILIZADO, É INFORMADO EMPRESA NA TELA)
          005 a 008 | 04 posições | Código do Estabelecimento, Código da Unidade ou Branco (NÃO UTILIZADO)
          009 a 012 | 04 posições | Código da Classificação contábil (Código da Filial)
          013 a 032 | 20 posições | Classificação Contábil Estendidada (NÃO UTILIZADO)
          033 a 035 | 03 posições | Item Contábil (NÃO UTILIZADO)
          036 a 075 | 40 posições | Descrição do Item Contábil (NÃO UTILIZADO)
          076 a 076 | 01 posições | Típo de Item: 1- Contábil ou 2 -Estático  (NÃO UTILIZADO)
          077 a 084 | 08 posições | Data do Lançamento no formato AAAAMMDD
          085 a 104 | 20 posições | Conta de débito
          105 a 124 | 20 posições | Conta de crédito
          125 a 128 | 04 posições | Código do Histórico Padrão
          129 a 129 | 01 posições | Se valor do lançamento negativo ( - ), senão branco -> Sinal colocado diretamente no servidor, na contrapartida.
          130 a 144 | 15 posições | Valor da partida (sem seperador decimal) -> Para pegar casa decimal foi dividido o valor por 10
          145 a 152 | 08 posições | Classificação Contábil c/ mais de 4 dígitos (NÃO UTILIZADO)
          153 a 172 | 20 posições | Código Auxiliar do Estabelecimento (NÃO UTILIZADO)
         }
    */
    
}