<?php
namespace Teknisa\CTB\Services;

use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\CTB\Factories\Ctb90100SpedContabilFactory;

class Ctb90100SpedContabilService {

    protected $entityManager;
    protected $environment;
    protected $newCode;
    protected $Ctb90100SpedContabilFactory;

    public function __construct(EntityManager $entityManager, Environment $environment, NewCode $newCode, Ctb90100SpedContabilFactory $Ctb90100SpedContabilFactory)
    {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->newCode  = $newCode;
        $this->Ctb90100SpedContabilFactory  = $Ctb90100SpedContabilFactory;
    }

    /*
     * Função para criar um arquivo a ser exportado.
     */
    public function createFileExportacao($fileName, $data, $columnName = "DSREGARQMAG") {
        // Verifica se algum dado foi retornado, lançando uma exceção caso contrário.
        if(empty($data)) {
            throw new \Exception("Não há dados a ser exibidos.");
        }

        // Verifica se há uma extensão já informada.
        if(empty(pathinfo($fileName, PATHINFO_EXTENSION)) || substr($fileName, -1) == '.') {
            // Se o nome do arquivo for terminado em '.', termina a extensão '.txt'.
            if (substr($fileName, -1) == '.') {
                $fileName .= "txt";
            }
            // Senão, adiciona a extensão '.txt' ao nome do arquivo.
            else {
                $fileName .= ".txt";
            }
        }
        // Cria ou sobrepõe um arquivo com o nome dado.
        try {
            $file = fopen(sys_get_temp_dir() . DIRECTORY_SEPARATOR . $fileName, "w");
           
            if (strpos($fileName, '../') !== false){
                die("ERRO");
            }
            
        } catch (\Exception $e) {
            // Se houver algum erro ao tentar abrir o arquivo, lança uma exceção.
            throw new \Exception("Ocorreu um erro ao criar o arquivo para exportação.");
        }
        // Se houver algum erro após tentar abrir o arquivo, lança uma exceção.
        if (!$file) {
            throw new \Exception("Ocorreu um erro ao criar o arquivo para exportação.");
        }

        // Inicializa a variável com o texto a ser impresso no arquivo.
        $text = "";
        // Percorre os dados, concatenando na variável a ser impressa.
        foreach ($data as $dataRow) {
            $text .= $dataRow["DSREGARQMAG"] . "\r\n";
        }

        // Imprime o texto no arquivo.
        $text = iconv("UTF-8", "WINDOWS-1252", $text);
         
        fprintf($file, "%s", $text);

        // Fecha o arquivo após as alterações.
        fclose($file);
        
        // Retorna o nome do arquivo com suas possíveis modificações.
        return $fileName;
    }
    
    
    private function createFileSpedContabil($row){
        // Cria os parâmetros para buscar os dados gerados na procedure.
        $params = array(
            //"NRORG" => $row["NRORG"],
            "NRSEQARQMAG" => $row["NRSEQARQMAG"]
        );
        $spedContabilData = $this->Ctb90100SpedContabilFactory->getSpedContabilData($params);

        // Chama a função para a criação do arquivo.
        $fileName = $this->createFileExportacao($row["NMARQUIVO"], $spedContabilData, "DSREGARQMAG");

        // Retorna o nome do arquivo exportado.
        return $fileName;
    }
    
    /*
     * Função para retornar no próximo sequencial dos arquivos de exporação.
     */
    public function getNextNrseqarqmag($nrorg) {
        return $this->newCode->getCodeByTableName('ARQMAGNETICO', $nrorg);
    }
    
    public function exportaSpedContabil($row) {
        // Valida os parâmetros associados.
        $this->validateSped($row);

        $nrorg = $this->environment->getNrorg();
        $row["NRSEQARQMAG"] = $this->newCode->getCodeByTableName('ARQMAGNETICO', $nrorg);

        // Monta o array com os parâmetros para chamar a procedure.
        $params = array(
            
            
            "NRSEQARQMAG" => $row["NRSEQARQMAG"],
            "CDEMPRESA" => $row["CDEMPRESA"],
            "DTINICIAL" => $row["DTINICIAL"],
            "DTFINAL" => $row["DTFINAL"],
            "DSNATUREZA" => $row["DSNATUREZA"],
            "IDSPED" => $row["IDSPED"],
            "PORTE" => $row["PORTE"],
            "GERARAZAO" => $row["GERARAZAO"],
            "TIPOLIVRO" => $row["TIPOLIVRO"],
            "FINALIDADE" => $row["FINALIDADE"],
            "PERIODO" => $row["PERIODO"],
            "MUDPLANCTB" => $row["MUDPLANCTB"],
            "CODPLANCTB" => $row["CODPLANCTB"],
            "DEMONSTANUAL" => $row["DEMONSTANUAL"]
            
        );

        // Chama a procedure de exportação.
        $this->Ctb90100SpedContabilFactory->callSpedContabilProcedure($params);
        

        // Foi evitado utilizar um bloco finally devido a possíveis
        // incompatibilidades com a versão do PHP.
        try{
            // Seta os parâmetros para exclusão dos dados temporários.
            $params = array(
                "NRORG" => $row["NRORG"],
                "NRSEQARQMAG" => $row["NRSEQARQMAG"]
            );
            // Chama a função para gerar o arquivo texto.
            $fileName = $this->createFileSpedContabil($row);

            // Retorna o nome do arquivo com suas possíveis modificações.
            return $fileName;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    private function validateSped($row) {

    
        if (empty($row["IDSPED"])) {
            throw new \Exception("Tipo de arquivo deve ser informado.");
        }
        if (empty($row["PERIODO"])) {
            throw new \Exception("Encerramento deve ser informado.");
        }
        if (empty($row["CDEMPRESA"])) {
            throw new \Exception("Empresa deve ser informada.");
        }
        if (empty($row["DTINICIAL"])) {
            throw new \Exception("Período de referência deve ser informado.");
        }
        if (empty($row["DTFINAL"])) {
            throw new \Exception("Período de referência deve ser informado.");
        }
        if (empty($row["DSNATUREZA"])) {
            throw new \Exception("Natureza do livro deve ser informado.");
        }
        if (empty($row["PORTE"])) {
            throw new \Exception("Empresa grande porte deve ser informada.");
        }
        if (empty($row["GERARAZAO"])) {
            throw new \Exception("Razão Auxiliar obrigatória deve ser informada.");
        }
        if (empty($row["FINALIDADE"])) {
            throw new \Exception("Finalidade Escrituração deve ser informada.");
        }
        if (empty($row["FINALIDADE"])) {
            throw new \Exception("Sub-Release deve ser informada.");
        }

        $nrorg = $this->environment->getNrorg();
        $row["NRORG"] = $nrorg;
        
    }
}