<?php

namespace Teknisa\CTB\Services;

use Teknisa\CTB\Factories\Ger17700RendimentosSociosFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;

class Ger17700RendimentosSociosService {

    protected $entityManager;
    protected $environment;
    protected $nrorg;
    protected $factory;
    protected $newCode;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, Environment $environment, Ger17700RendimentosSociosFactory $factoryRendimento, NewCode $newCode) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->nrorg = $environment->getNrorgTrab();
        $this->factory = $factoryRendimento;
        $this->newCode = $newCode;
    }

    public function generateSequentialCode($tableName) {
        return $this->newCode->getCodeByTableName($tableName, $this->nrorg, 1, 0, true);
    }

    public function deleteSocioChildren($param) {
        $this->factory->deleteSocioChildren(array ("NRSEQREG" => $param, "NRORG" => $this->nrorg));
    }

    public function validateSocio($row) {

        if (!\Zeedhi\Framework\Util\Functions::arrayKeyExists('NMREG', $row) || !\Zeedhi\Framework\Util\Functions::arrayKeyExists('CDREG', $row)
            || !\Zeedhi\Framework\Util\Functions::arrayKeyExists('CDPAISREG', $row) || !\Zeedhi\Framework\Util\Functions::arrayKeyExists('IDTIPOREG', $row))
            throw new \Exception('Campos obrigatórios vazios!');
        if (ctype_space($row['NMREG']) || ctype_space($row['IDTIPOREG']))
            throw new \Exception('Campos obrigatórios vazios!');

    }

    public function validateRelacao($row) {
        if (!\Zeedhi\Framework\Util\Functions::arrayKeyExists('CDEMPRESA', $row) || !\Zeedhi\Framework\Util\Functions::arrayKeyExists('NRSEQREG', $row)
            || !\Zeedhi\Framework\Util\Functions::arrayKeyExists('IDQUALIY600', $row) || !\Zeedhi\Framework\Util\Functions::arrayKeyExists('DTALTSOC', $row))
            throw new \Exception('Campos obrigatórios vazios!');
        if (ctype_space($row['CDEMPRESA']) || is_null($row['NRSEQREG'])
            || ctype_space($row['IDQUALIY600']) || ctype_space($row['DTALTSOC']))
            throw new \Exception('Campos obrigatórios vazios!');
        $relacaoExists = $this->factory->verificaRelacaoSocioEmpresa(array(
          "CDEMPRESA" => $row['CDEMPRESA'],
          "NRSEQREG" => $row['NRSEQREG'],
          "NRORG" => $this->nrorg
        ));
        if ($relacaoExists) {
            throw new \Exception("O colaborador selecionado já está relacionado a esta empresa.");
        }
    }

    public function validateRendimento($row) {

        if (!\Zeedhi\Framework\Util\Functions::arrayKeyExists('CDEMPRESA', $row) || !\Zeedhi\Framework\Util\Functions::arrayKeyExists('NRSEQREG', $row)
            || !\Zeedhi\Framework\Util\Functions::arrayKeyExists('IDQUALIY611', $row)|| !\Zeedhi\Framework\Util\Functions::arrayKeyExists('DTREFERENCIA', $row))
            throw new \Exception('Campos obrigatórios vazios!');
        if (ctype_space($row['CDEMPRESA']) || is_null($row['NRSEQREG'])
            || ctype_space($row['IDQUALIY611']) || ctype_space($row['DTREFERENCIA']))
            throw new \Exception('Campos obrigatórios vazios!');
        $rendimentoAnoEmpresaExists = $this->factory->verificaRendimentoAnoEmpresa(array(
          "CDEMPRESA" => $row['CDEMPRESA'],
          "DTREFERENCIA" => $row['DTREFERENCIA'],
          "NRSEQREG" => $row['NRSEQREG'],
          "NRORG" => $this->nrorg
        ));
        if ($rendimentoAnoEmpresaExists && !$row['anoUnchanged']) {
            throw new \Exception("O rendimento deste sócio neste ano para a empresa selecionada já foi cadastrado.");
        }
    }




}
